#!/usr/bin/perl -w
use POSIX;
use Getopt::Long;
use File::Basename;
use DBI;                                                                      
#use strict;
use vars qw( $schichten $nu_template
             $show_help $show_version );
	     
my $version = "1.0.1";
my $schichten = 1;
my $nu_template = "/etc/ehrlich/schicht.prn";
my $logfile = "";


&GetOptions(
    "schichten=i"=> \$schichten,
    "template=s" => \$nu_template,
    "help"       => \$show_help, 
    "version"    => \$show_version,
);

$logfile = $ARGV[0];
&usage("Die Datei fehlt") if ( $logfile eq "" );
&usage("") if ($show_help);

if ($show_version) {
    print <<".";
$0 version $version
by  Konrad Riedel <k.riedel\@gmx.de>
.
    exit(0);
}
#stdin, wenn keine Datei

&catch_signals;
$i=-312;
open(TEMPL,"<$nu_template");
while (<TEMPL>) {
  print;
  if ($i++ == 0) {
    #warn("$_\n");
    $bs = basename($logfile);
    $d = dirname($logfile);
    $schicht = substr($bs,11,1);
    #Schichtbeginn 5.30 13.30 22.30 in gmtime?
    $zt=mktime (0,30,$schicht*8-3, substr($bs,9,2), substr($bs,7,2)-1, substr($bs,3,4)-1900,0,0,-1);                                  
    $Datum = strftime ("%A, %d.%m.%Y ",localtime($zt));
    print " 20 0 rmoveto ($Datum $schicht.Schicht) show \n";
    &diagramm;
    print "\n";
  };
  
}
close(TEMPL);

exit 0;

#zeichnet eine Logdatei 
sub paint_log {
  my $logfile = $_[0];	
  open(LOG,"<$logfile");
  warn "$logfile\n"; 	
  while (<LOG>) {
    chop;
    ($zeit,$mlog)=split ;
    $x=195+($zeit - $zt)/10/$schichten;
    for ($m=0;$m<15;$m++) {        #jede Maschine
      $y=220+$m*50;
      #warn substr($mlog,$m,1),"\n";
      print " $x $y M St stroke " if substr($mlog,$m,1);
    }
  }
  my $w = "Ende: " . localtime($zeit);
  warn "$w";
  close(LOG);
}

sub diagramm {
  print " save ";
  print " /Kr {10 10 rlineto 0 -20 rlineto -10 10 rlineto } def ";
  print " /St {0 10 rlineto } def ";
  print " /RL {rlineto} def ";
 
  print "1 setlinewidth 0.99 setgray ";
  #Schleife ber alle Maschinen 
  for ($m=0;$m<15;$m++) {       #jede 2.Zeile grau hinterlegen
    $y=220-23+$m*50;
    #print " 100 $y M 3380 0 RL 0 50 RL -3380 0 RL 0 -50 RL fill " if ($m % 2); 
    print " 65 $y M 120 0 RL 0 50 RL -120 0 RL 0 -50 RL fill " if ($m % 3); 
    print " 3080 $y M 320 0 RL 0 50 RL -320 0 RL 0 -50 RL fill " if ($m % 3); 
  }
  #alle n Minuten 
  #6 Pixel auf 1 Minute (360 je Stunde)
  #print " /Helvetica findfont 10 scalefont setfont ";
  print " 1 setlinewidth 0 setgray ";
  if ($schichten>20) {
    for ($k=0;$k<$schichten*8;$k+=24) {
      $x=195+($k+0.5)*360/$schichten;
      print " $x 200 M 0 750 rlineto stroke \n" ; # ijf ($k % 360 == 0); 
      #Tageslinie 
      $st = strftime ("%d %a",localtime($zt+$k*3600)) ;
      print " ; : 0 0 3437 2429 rc 0 0 0 sco F0_33 Ji";
      print " $x 950 M ($st) dup stringwidth pop 2 div neg 0 rmoveto show " if ($k%48);
    }

  } else {
    for ($k=-0.5;$k<$schichten*8-0.5;$k+=0.5) {
      $x=195+($k+0.5)*360/$schichten;
      print " $x 200 M 0 750 rlineto stroke \n" ; # ijf ($k % 360 == 0); 
      # HalbStundenlinie ;
      $st = ($schicht*8-2+$k) % 24;
      print " ; : 0 0 3437 2429 rc 0 0 0 sco F0_33 Ji"; 
      print " $x 950 M ($st) dup stringwidth pop 2 div neg 0 rmoveto show " if (int($k) == $k);
    }
  }
  print " 1 setlinewidth 0 setgray ";
  my $ztschichtanfang = $zt;
  my $cnts=$schicht;
  for(1..$schichten){
     &paint_log($d .'/'. $bs );
     #Schicht weiterzhlen
     $ztschichtanfang += 8*60*60;
     $cnts = ($cnts % 3) + 1;
     $bs = strftime ("mde%Y%m%d",localtime($ztschichtanfang)) . "$cnts.log";
  }

  $dbh = DBI->connect("DBI:mysql:ehrlich", "root", "");
  my $tag = strftime ("%Y-%m-%d",localtime($zt));
  my($IO,$Zykluszeit,$Zyklen,$ZyGesamt,$UntGesamt);
  
  $dbh->{RaiseError} = 1; # do this, or check every call for errors
  
  if ($schichten > 1) {
    my $tagE = strftime ("%Y-%m-%d",localtime($ztschichtanfang));
    warn "WHERE tag BETWEEN '$tag' AND '$tagE'\n";
    $sth = $dbh->prepare("SELECT maschine, AVG(zykluszeit),SUM(schuss),
                          SUM(sLauf), SUM(sStand)  
  			  FROM mdelog WHERE tag BETWEEN '$tag' AND '$tagE'
			  GROUP BY maschine");
  } else {
    warn "WHERE tag='$tag' AND schicht=$schicht\n";
    $sth = $dbh->prepare("SELECT maschine,zykluszeit,schuss,sLauf,sStand 
                          FROM mdelog WHERE tag='$tag' AND schicht=$schicht");
  }
  $sth->execute;
  $sth->bind_columns(undef,\($IO,$Zykluszeit,$Zyklen,$ZyGesamt,$UntGesamt));
  
  while($sth->fetch) {
    $y=210+$IO*50; 
    print " 3180 $y M (";
    printf "%3.1f",$Zykluszeit if defined($Zykluszeit);
    print ") dup stringwidth pop neg 0 rmoveto show " ;
    print " 3300 $y M ($Zyklen) dup stringwidth pop neg 0 rmoveto show " ;
    $Auslastung=0;
    $ZyGesamt = defined($Zykluszeit) ? $Zyklen * $Zykluszeit : 0;
    $UntGesamt = 28800 * $schichten - $ZyGesamt;
    $sZyGesamt += $ZyGesamt;
    $sUntGesamt += $UntGesamt; 
    if ($ZyGesamt+$UntGesamt) {$Auslastung=$ZyGesamt/($ZyGesamt+$UntGesamt)};
    $Auslastung = int($Auslastung*100);
    print " 3390 $y M ($Auslastung) dup stringwidth pop neg 0 rmoveto show " ; 

  }
 $Auslastung=0;$y=210+15*50; 
 if ($sZyGesamt+$sUntGesamt) {$Auslastung=$sZyGesamt/($sZyGesamt+$sUntGesamt)}; 
 $Auslastung=int($Auslastung*100);                                           
 print " 3390 $y M ($Auslastung) dup stringwidth pop neg 0 rmoveto show " ;
 #print "$IO,$Zykluszeit,$Zyklen,$ZyGesamt,$UntGesamt\n";
  print "restore ";  
}

sub usage {
  local($mess) = @_;

  print "$0: $mess\n\n";

  print <<"_EOF_";
Benutzung:   [-options ...] Datei (mdejjjjmmtts.log - mde199903181.log)
                                   
where options include:                            Defaults -
    -template      file.ps     Vorlage            /etc/ehrlich/schicht.prn
    -schichten	   1..90       6 fr WE		  1
    
Alle Optionen knnen mit einem Buchstaben abgekrzt werden.
 
_EOF_

 exit 7;
}

sub catch_signals {
  $SIG{'INT'} = 'SIGNAL_CLEANUP';
  $SIG{'HUP'} = 'SIGNAL_CLEANUP';
  $SIG{'QUIT'} = 'SIGNAL_CLEANUP';
  $SIG{'PIPE'} = 'SIGNAL_CLEANUP';
  $SIG{'ALRM'} = 'SIGNAL_CLEANUP';
}

sub cleanup {
  local($message,$exit_status) = @_;

  print "$0: $message\n";
  exit $exit_status;
}

sub SIGNAL_CLEANUP {

  print "\n$0: interrupted!\n";
  exit 2;
}

__END__

=head1 NAME

schicht - Ausgabe des Schichtprotokolls                                     

=head1 SYNOPSIS                 

  einfach starten

=head1 DESCRIPTION

  

=head1 SEE ALSO


=cut                                              
 

